/**
 *  GeoNetworking over IPv6 source pseudo layer. It forwards a BTP message to upper layer every second
 *  SHALL NOT BE INCLUDED IN RELEASE
 *				
 *  @author	 ETSI / STF424
 *  @version	$URL: svn+ssh://vcs.etsi.org/TTCN3/TOOL/ITS_TestPlatform/trunk/javasrc/adapter/org/etsi/its/adapter/layers/test/CamSourceLayer.java $
 *			  $Id: CamSourceLayer.java 146 2011-04-21 11:33:57Z berge $
 *
 */
package org.etsi.its.adapter.layers.test;

import java.util.Stack;

import org.etsi.its.adapter.IManagementLayers;

/**
 *  GeoNetworking over IPv6 source pseudo layer. It forwards a BTP message to upper layer every second
 */
public class Gn6SourceLayer extends SourceLayer {

	/**
	 * Constructor
	 * @param  management   Layer management instance
	 * @param  lowerStack   Lower protocol stack   
	 */
	public Gn6SourceLayer(IManagementLayers management, Stack<String> lowerStack) {
		super(management, lowerStack);
		// message =
	}

	/**
	 * AcGn6InterfaceInfoList message
	 */
	@SuppressWarnings("unused")
	private byte[] _acGn6InterfaceInfoList = new byte[]{
		(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x03, // Number of items
			(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x0c, // Length of the interface name #0
			(byte)0x49, (byte)0x6e, (byte)0x74, (byte)0x65, (byte)0x72, (byte)0x66, (byte)0x61, (byte)0x63, (byte)0x65, (byte)0x20, (byte)0x23, (byte)0x30, 
			(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x03, // Number of Ipv6Address
				(byte)0x10, (byte)0x01, (byte)0x06, (byte)0x18, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x51, (byte)0x99, (byte)0xcc, (byte)0x70, // Ipv6Address #1
				(byte)0x10, (byte)0x01, (byte)0x06, (byte)0x18, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x51, (byte)0x99, (byte)0xcc, (byte)0x71, // Ipv6Address #2
				(byte)0x10, (byte)0x01, (byte)0x06, (byte)0x18, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x51, (byte)0x99, (byte)0xcc, (byte)0x72, // Ipv6Address #3
			(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x0c,  // Length of the interface name #1
			(byte)0x49, (byte)0x6e, (byte)0x74, (byte)0x65, (byte)0x72, (byte)0x66, (byte)0x61, (byte)0x63, (byte)0x65, (byte)0x20, (byte)0x23, (byte)0x31, 
			(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x03, // Number of Ipv6Address
				(byte)0x20, (byte)0x01, (byte)0x06, (byte)0x18, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x51, (byte)0x99, (byte)0xcc, (byte)0x70, // Ipv6Address #1
				(byte)0x20, (byte)0x01, (byte)0x06, (byte)0x18, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x51, (byte)0x99, (byte)0xcc, (byte)0x71, // Ipv6Address #2
				(byte)0x20, (byte)0x01, (byte)0x06, (byte)0x18, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x51, (byte)0x99, (byte)0xcc, (byte)0x72, // Ipv6Address #3
			(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x0c, // Length of the interface name #2
			(byte)0x49, (byte)0x6e, (byte)0x74, (byte)0x65, (byte)0x72, (byte)0x66, (byte)0x61, (byte)0x63, (byte)0x65, (byte)0x20, (byte)0x23, (byte)0x32, 
			(byte)0x00, (byte)0x00, (byte)0x00, (byte)0x03, // Number of Ipv6Address
				(byte)0x30, (byte)0x01, (byte)0x06, (byte)0x18, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x51, (byte)0x99, (byte)0xcc, (byte)0x70, // Ipv6Address #1
				(byte)0x30, (byte)0x01, (byte)0x06, (byte)0x18, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x51, (byte)0x99, (byte)0xcc, (byte)0x71, // Ipv6Address #2
				(byte)0x30, (byte)0x01, (byte)0x06, (byte)0x18, (byte)0x04, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x00, (byte)0x51, (byte)0x99, (byte)0xcc, (byte)0x72 // Ipv6Address #3
	};

} // End of class Gn6SourceLayer
